/*
    ES Knot
    Copyright 2005, 2006 Martin Storsj

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Martin Storsj
    martin@martin.st
*/

#ifndef __KNOT_H
#define __KNOT_H

#include <GLES/gl.h>

#include "glmath.h"

#define RINGS 70
#define POINTS 8
#define SINETABLESIZE 512
#define SINETABLEMASK 0x01FF
#define SINETABLEMASKSIZE 9

class Knot {
public:
	Knot(int w, int h);
	~Knot();
	void draw(unsigned int msec);
	void reinit();
	void setSize(int w, int h);

	enum TexMode { UNTEXTURED, PLAINTEX, TEXSPEC, CELSHADED };
	void setTexMode(TexMode mode);
	void setUpdateVertices(bool update);
	void setMotionBlur(bool blur);
	void setRadialBlur(bool blur);
	void setRadialGlow(bool glow);

	void startMorph();

private:
	TYPE vertices[3*(RINGS+1)*(POINTS+1)];
	TYPE normals[3*(RINGS+1)*(POINTS+1)];
	TYPE texcoords[2*(RINGS+1)*(POINTS+1)];
	TYPE celcoords[2*(RINGS+1)*(POINTS+1)];
	GLushort indices[6*RINGS*POINTS];
#define WIREFRAMESIZE (RINGS*(POINTS+1) + 1 + (RINGS+1)*(POINTS-1))
	GLushort wireframe[WIREFRAMESIZE];
	TYPE ringvertices[4*POINTS];
	TYPE ringcoords23[16*RINGS];
	TYPE ringcoords25[16*RINGS];
	TYPE *ringcoords;
	TYPE ringcoordstemp[16*RINGS];

	static void polar(float pos, TYPE *vect, int P, int Q);
	void precalc();
	void genTexture();
	void genCelTexture();
	GLuint genEmptyTexture(int &w, int &h);
	void updateCelCoords(TYPE *verts, TYPE *norms);
	void getLightInverse(TYPE* light);
	void calcDynVertices();
	void calcDynVerticesNormals(TYPE *verts, TYPE *norms, const TYPE *coords);
	void makeRingCoords(TYPE *coords, int P, int Q);

	TYPE fastSin(TYPE x);
	void setDrawState();
	void morphMatrices(TYPE *out, const TYPE *first, const TYPE *second, TYPE pos);

	void reinit(TexMode mode);
	int width, height;

	unsigned int lasttime;
	bool morphing;
	unsigned int morphStart;

	GLuint texture;
	GLuint celTexture;
	TexMode texMode;
	TYPE texOffsetX, texOffsetY;

	bool updateBumps;
	TYPE bumpOffset;

	TYPE sinTable[SINETABLESIZE];

	bool motionBlur;
	GLuint motionTexture;
	int motionWidth, motionHeight;

	bool radialBlur;
	bool glow;
	GLuint radialTexture;
	int radialWidth, radialHeight;

	GLbitfield (APIENTRY *ext_glQueryMatrixxOES)(GLfixed mantissa[16], GLint exponent[16]);
};

#endif
